# Tcl based calculation using an index accessor


# this coroutine adapts the index interface into a coroutine
proc get_from_record_index { the_record } {
    set vlen [ $the_record size ]
    yield [ info coroutine ]
    for { set idx 0 } { $idx < $vlen } { incr idx } {
        yield [ $the_record value $idx ]
    }
    return -code break
}

# this coroutine adapts the list interface into a coroutine
proc get_from_record_list { the_record } {
    yield [ info coroutine ]
    foreach value $the_record  {
        yield $value
    }
    return -code break
}

#this calculation engine is independent of the iteration format due to the use of a coroutine
proc do_calculation { data_fetcher } {

  set wval 0.0
  set old_val 0.0
  set vlen 0
  while 1 {
    set value [ data_fetcher ]
    set wval [ expr { pow(1.009 + sqrt($old_val)*(1/(abs($wval)+0.001) + $value ), 1.0002) } ]
    set old_val [ expr { $wval/2.355832 } ]
    incr vlen
  }
  return [ expr { $wval/$vlen } ]
}



#This function sets up the coroutine that is specific to the index interface
proc do_calculation_index record {
  coroutine data_fetcher get_from_record_index $record
  do_calculation data_fetcher
}

#This function sets up the coroutine that is specific to the list interface
proc do_calculation_list record {
  coroutine data_fetcher get_from_record_list $record
  do_calculation data_fetcher
}

